==> moonlight-1.20-2.16.14-fabric-sources.jar: net/mehvahdjukaar/moonlight/core/Moonlight.java <==
package net.mehvahdjukaar.moonlight.core;

import net.mehvahdjukaar.moonlight.api.MoonlightRegistry;
import net.mehvahdjukaar.moonlight.api.events.IDropItemOnDeathEvent;
import net.mehvahdjukaar.moonlight.api.events.MoonlightEventsHelper;
import net.mehvahdjukaar.moonlight.api.fluids.FluidContainerList;
import net.mehvahdjukaar.moonlight.api.fluids.SoftFluid;
import net.mehvahdjukaar.moonlight.api.fluids.SoftFluidRegistry;
import net.mehvahdjukaar.moonlight.api.integration.HardcodedBlockTypes;
import net.mehvahdjukaar.moonlight.api.item.additional_placements.AdditionalItemPlacementsAPI;
import net.mehvahdjukaar.moonlight.api.map.MapDataRegistry;
import net.mehvahdjukaar.moonlight.api.misc.DynamicHolder;
import net.mehvahdjukaar.moonlight.api.misc.EventCalled;
import net.mehvahdjukaar.moonlight.api.misc.RegistryAccessJsonReloadListener;
import net.mehvahdjukaar.moonlight.api.platform.PlatHelper;
import net.mehvahdjukaar.moonlight.api.platform.RegHelper;
import net.mehvahdjukaar.moonlight.api.resources.pack.DynResourceGenerator;
import net.mehvahdjukaar.moonlight.api.resources.recipe.BlockTypeSwapIngredient;
import net.mehvahdjukaar.moonlight.api.set.BlockSetAPI;
import net.mehvahdjukaar.moonlight.api.set.leaves.LeavesTypeRegistry;
import net.mehvahdjukaar.moonlight.api.set.wood.WoodType;
import net.mehvahdjukaar.moonlight.api.set.wood.WoodTypeRegistry;
import net.mehvahdjukaar.moonlight.api.trades.ItemListingRegistry;
import net.mehvahdjukaar.moonlight.api.util.DispenserHelper;
import net.mehvahdjukaar.moonlight.core.fluid.SoftFluidInternal;
import net.mehvahdjukaar.moonlight.core.map.MapDataInternal;
import net.mehvahdjukaar.moonlight.core.misc.VillagerAIInternal;
import net.mehvahdjukaar.moonlight.core.network.ModMessages;
import net.mehvahdjukaar.moonlight.core.set.BlockSetInternal;
import net.mehvahdjukaar.moonlight.core.set.BlocksColorInternal;
import net.mehvahdjukaar.moonlight.core.set.DebugBlockTypes;
import net.minecraft.class_1657;
import net.minecraft.class_1661;
import net.minecraft.class_1792;
import net.minecraft.class_1799;
import net.minecraft.class_1802;
import net.minecraft.class_1806;
import net.minecraft.class_1928;
import net.minecraft.class_22;
import net.minecraft.class_2960;
import net.minecraft.class_3218;
import net.minecraft.class_3264;
import net.minecraft.class_5455;
import net.minecraft.class_7923;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.jetbrains.annotations.ApiStatus;

import java.lang.ref.WeakReference;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

@ApiStatus.Internal
public class Moonlight {

    public static final String MOD_ID = "moonlight";

    public static final Logger LOGGER = LogManager.getLogger("Moonlight");
    public static final boolean HAS_BEEN_INIT = true;

    public static final ThreadLocal<WeakReference<class_5455>> EARLY_REGISTRY_ACCESS = new ThreadLocal<>();

    public static class_2960 res(String name) {
        return new class_2960(MOD_ID, name);
    }

    //called on mod creation
    public static void commonInit() {
        CommonConfigs.init();

        BlockSetInternal.registerBlockSetDefinition(WoodTypeRegistry.INSTANCE);
        BlockSetInternal.registerBlockSetDefinition(LeavesTypeRegistry.INSTANCE);
        //MoonlightEventsHelper.addListener( BlockSetInternal::addTranslations, AfterLanguageLoadEvent.class);
        HardcodedBlockTypes.init();
        MoonlightRegistry.init();

        ModMessages.init();

        BlockTypeSwapIngredient.init();
        VillagerAIInternal.init();
        MapDataInternal.init();
        SoftFluidInternal.init();
        RegHelper.addDynamicDispenserBehaviorRegistration(Moonlight::registerBuiltinFluidBehavior);

        PlatHelper.addCommonSetup(Moonlight::commonSetup);

        PlatHelper.addServerReloadListener(new ItemListingRegistry(), Moonlight.res("villager_trades"));

        //hack
        BlockSetAPI.addDynamicRegistration((reg, wood) -> AdditionalItemPlacementsAPI.afterItemReg(),
                WoodType.class, class_7923.field_41181);

        //client init
        if (PlatHelper.getPhysicalSide().isClient()) {
            MoonlightClient.initClient();
        }
    }

    private static void commonSetup() {
        BlocksColorInternal.setup();

        if (PlatHelper.isDev()) {
            //MixinEnvironment.getCurrentEnvironment().audit();
        }
        if (PlatHelper.getPhysicalSide().isClient()) {
            MoonlightClient.setupClient();
        }

        if (CommonConfigs.EXTRA_DEBUG.get()) {
            DebugBlockTypes.writeToFile();
        }
    }

    @EventCalled
    public static void onPlayerCloned(class_1657 oldPlayer, class_1657 newPlayer, boolean wasDeath) {
        if (wasDeath && !oldPlayer.method_37908().method_8450().method_8355(class_1928.field_19389)) {
            var inv = oldPlayer.method_31548();
            int i = 0;
            for (var v : inv.field_7547) {
                if (v != class_1799.field_8037) {
                    IDropItemOnDeathEvent e = IDropItemOnDeathEvent.create(v, oldPlayer, false);
                    MoonlightEventsHelper.postEvent(e, IDropItemOnDeathEvent.class);
                    if (e.isCanceled()) {
                        newPlayer.method_31548().method_5447(i, e.getReturnItemStack());
                    }
                }
                i++;
            }
        }
    }

    //by mixin, even earlier, needed for recipe manager. not needed in 1.21.1+
    public static void setServerRegistryAccess(class_5455.class_6890 registryAccess) {
        EARLY_REGISTRY_ACCESS.set(new WeakReference<>(registryAccess));
    }

    @EventCalled
    public static void afterDataReload(class_5455 registryAccess, boolean client) {
        EARLY_REGISTRY_ACCESS.set(new WeakReference<>(registryAccess));

        RegistryAccessJsonReloadListener.runReloads(registryAccess);
        DynResourceGenerator.clearAfterReload(class_3264.field_14190);
        DynamicHolder.clearCache();
        DispenserHelper.reload(registryAccess);
    }

    @EventCalled
    public static void beforeServerStart(class_5455 ra) {
        SoftFluidInternal.doPostInitServer(ra);
    }

    public static void assertInitPhase() {
        if (!PlatHelper.isInitializing()) {
            //TODO: re add once all mods are updated. we have to let fabric use its own initializer. this is too strict so we ignore
            if (PlatHelper.isDev() && PlatHelper.getPlatform().isForge()) {
                throw new AssertionError("Method has to be called during main mod initialization phase. Client and Server initializer are not valid, you must call in the main one");
            }
        }
    }

    public static class_22 getMapDataFromKnownKeys(class_3218 level, int mapId) {
        var d = level.method_17891(class_1806.method_17440(mapId));
        if (d == null) {
            d = level.method_17891("magicmap_" + mapId);
            if (d == null) {
                d = level.method_17891("mazemap_" + mapId);
            }
        }
        return d;
    }

    public static void checkDataPackRegistry() {
        try {
            SoftFluidRegistry.getEmpty();
            MapDataRegistry.getDefaultType();
        } catch (Exception e) {
            throw new RuntimeException("""
                    Not all required entries were found in datapack registry. How did this happen?
                    This MUST be some OTHER mod messing up datapack registries (currently Cyanide is known to cause this).
                    Note that this could be caused by Paper or similar servers. Know that those are NOT meant to be used with mods""", e);
        }
    }

    public static void crashIfInDev(String message) {
        if (PlatHelper.isDev()) throw new AssertionError(message);
        else {
            Moonlight.LOGGER.error(message);
        }
    }

    public static void crashIfInDev() {
        crashIfInDev("");
    }

    public static void logIfInDev(String s) {
        if (PlatHelper.isDev()) LOGGER.error(s);
    }

    public static void registerBuiltinFluidBehavior(DispenserHelper.Event event) {
        Set<class_1792> itemSet = new HashSet<>();
        for (SoftFluid f : SoftFluidRegistry.getRegistry(event.getRegistryAccess())) {
            Collection<FluidContainerList.Category> categories = f.getContainerList().getCategories();
            for (FluidContainerList.Category c : categories) {
                class_1792 empty = c.getEmptyContainer();
                //prevents registering stuff twice
                if (empty != class_1802.field_8162 && !itemSet.contains(empty)) {
                    event.register(new DispenserHelper.FillFluidHolderBehavior(empty));
                    itemSet.add(empty);
                }
                for (class_1792 full : c.getFilledItems()) {
                    if (full != class_1802.field_8162 && !itemSet.contains(full)) {
                        event.register(new DispenserHelper.FillFluidHolderBehavior(full));
                        itemSet.add(full);
                    }
                }
            }
        }
    }

    private static boolean warnedInvalidServer = false;

    public static void warnInvalidServer() {
        if (!warnedInvalidServer) {
            LOGGER.error("It seems like you are on a VANILLA server. This could cause issues and is NOT supported and you are OUT OF SUPPORT!");
            warnedInvalidServer = true;
        }

    }
}

==> moonlight-1.21-2.25.0-fabric-sources.jar: net/mehvahdjukaar/moonlight/core/Moonlight.java <==
package net.mehvahdjukaar.moonlight.core;

import net.mehvahdjukaar.moonlight.api.MoonlightRegistry;
import net.mehvahdjukaar.moonlight.api.events.IDropItemOnDeathEvent;
import net.mehvahdjukaar.moonlight.api.events.MoonlightEventsHelper;
import net.mehvahdjukaar.moonlight.api.fluids.FluidContainerList;
import net.mehvahdjukaar.moonlight.api.fluids.SoftFluid;
import net.mehvahdjukaar.moonlight.api.fluids.SoftFluidRegistry;
import net.mehvahdjukaar.moonlight.api.integration.HardcodedBlockTypes;
import net.mehvahdjukaar.moonlight.api.misc.*;
import net.mehvahdjukaar.moonlight.api.platform.PlatHelper;
import net.mehvahdjukaar.moonlight.api.platform.RegHelper;
import net.mehvahdjukaar.moonlight.api.platform.network.NetworkHelper;
import net.mehvahdjukaar.moonlight.api.set.BlockSetAPI;
import net.mehvahdjukaar.moonlight.api.set.leaves.LeavesType;
import net.mehvahdjukaar.moonlight.api.set.leaves.LeavesTypeRegistry;
import net.mehvahdjukaar.moonlight.api.set.wood.WoodTypeRegistry;
import net.mehvahdjukaar.moonlight.api.trades.ItemListingManager;
import net.mehvahdjukaar.moonlight.api.util.DispenserHelper;
import net.mehvahdjukaar.moonlight.core.fluid.SoftFluidInternal;
import net.mehvahdjukaar.moonlight.core.map.MapDataInternal;
import net.mehvahdjukaar.moonlight.core.misc.VillagerAIInternal;
import net.mehvahdjukaar.moonlight.core.network.ClientBoundSyncWorldDataMessage;
import net.mehvahdjukaar.moonlight.core.network.ModNetworking;
import net.mehvahdjukaar.moonlight.core.pack.DynamicResourcesInternals;
import net.mehvahdjukaar.moonlight.core.set.BlockSetInternal;
import net.mehvahdjukaar.moonlight.core.set.BlocksColorInternal;
import net.mehvahdjukaar.moonlight.core.set.DebugBlockTypes;
import net.minecraft.class_1657;
import net.minecraft.class_1661;
import net.minecraft.class_1792;
import net.minecraft.class_1799;
import net.minecraft.class_1802;
import net.minecraft.class_1928;
import net.minecraft.class_1937;
import net.minecraft.class_2248;
import net.minecraft.class_2960;
import net.minecraft.class_3222;
import net.minecraft.class_3264;
import net.minecraft.class_5455;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.jetbrains.annotations.ApiStatus;

import java.lang.ref.WeakReference;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

@ApiStatus.Internal
public class Moonlight {

    public static final String MOD_ID = "moonlight";

    public static final Logger LOGGER = LogManager.getLogger("Moonlight");
    public static final boolean HAS_BEEN_INIT = true;

    public static final ThreadLocal<WeakReference<class_5455>> EARLY_REGISTRY_ACCESS = new ThreadLocal<>();

    private static final Set<String> DEPENDENTS = new HashSet<>();
    private static boolean verboseLogging = false;

    public static class_2960 res(String name) {
        return class_2960.method_60655(MOD_ID, name);
    }

    //called on mod creation
    public static void commonInit() {
        CommonConfigs.init();

        BlockSetInternal.registerBlockSetDefinition(WoodTypeRegistry.INSTANCE);
        BlockSetInternal.registerBlockSetDefinition(LeavesTypeRegistry.INSTANCE);
        //MoonlightEventsHelper.addListener( BlockSetInternal::addTranslations, AfterLanguageLoadEvent.class);
        HardcodedBlockTypes.init();
        MoonlightRegistry.init();
        ItemListingManager.init();

        ModNetworking.init();

        DynamicResourcesInternals.init();
        VillagerAIInternal.init();
        MapDataInternal.init();
        SoftFluidInternal.init();
        RegHelper.addDynamicDispenserBehaviorRegistration(Moonlight::registerBuiltinFluidBehavior);

        PlatHelper.addCommonSetup(Moonlight::commonSetup);
        PlatHelper.addReloadableCommonSetup(Moonlight::afterDataReloadOrDataSync);

        PlatHelper.addServerReloadListener(ItemListingManager::new, Moonlight.res("villager_trade"));

        //client init
        if (PlatHelper.getPhysicalSide().isClient()) {
            MoonlightClient.initClient();
        }

        BlockSetAPI.addDynamicBlockRegistration(Moonlight::ensureBlockSetsInitialized, LeavesType.class);
    }

    //dumb
    private static void ensureBlockSetsInitialized(Registrator<class_2248> blockRegistrator, Collection<LeavesType> blockTypes) {
    }


    private static void commonSetup() {

        BlocksColorInternal.setup();
        if (PlatHelper.getPhysicalSide().isClient()) {
            MoonlightClient.setupClient();
        }

        if (CommonConfigs.EXTRA_DEBUG.get() || PlatHelper.isDev()) {
            DebugBlockTypes.writeToFile();
        }
    }

    @EventCalled
    public static void onPlayerCloned(class_1657 oldPlayer, class_1657 newPlayer, boolean wasDeath) {
        if (wasDeath && !oldPlayer.method_37908().method_8450().method_8355(class_1928.field_19389)) {
            var inv = oldPlayer.method_31548();
            int i = 0;
            for (var v : inv.field_7547) {
                if (v != class_1799.field_8037) {
                    IDropItemOnDeathEvent e = IDropItemOnDeathEvent.create(v, oldPlayer, false);
                    MoonlightEventsHelper.postEvent(e, IDropItemOnDeathEvent.class);
                    if (e.isCanceled()) {
                        newPlayer.method_31548().method_5447(i, e.getReturnItemStack());
                    }
                }
                i++;
            }
        }
    }

    @EventCalled
    public static void onDataSyncToPlayer(class_3222 player, boolean joined) {
        //send syncing packets just on login. datapack registries don't change on reload
        if (joined) {
            SoftFluidInternal.onDataSyncToPlayer(player, true);
        }
        class_1937 world = player.method_37908();
        for (var type : MoonlightRegistry.WORLD_SAVED_DATA_TYPE_REGISTRY) {
            WorldSavedData data = type.getData(world);
            NetworkHelper.sendToClientPlayer(player, new ClientBoundSyncWorldDataMessage<>(data));
        }
    }

    //by mixin, even earlier, needed for recipe manager. not needed in 1.21.1+
    public static void setServerRegistryAccess(class_5455.class_6890 registryAccess) {
        EARLY_REGISTRY_ACCESS.set(new WeakReference<>(registryAccess));
    }

    @EventCalled
    private static void afterDataReloadOrDataSync(class_5455 registryAccess, boolean client) {
        EARLY_REGISTRY_ACCESS.set(new WeakReference<>(registryAccess));
        RegistryAccessJsonReloadListener.runReloads(registryAccess);
        DynamicResourcesInternals.clearAfterReload(class_3264.field_14190);
        DynamicHolder.clearCache();

        HolderReference.clearCache();
        DispenserHelper.reload(registryAccess, client);
    }

    @EventCalled
    public static void beforeServerStart(class_5455 ra) {
        SoftFluidInternal.doPostInitServer(ra);
    }

    public static void assertInitPhase() {
        if (!PlatHelper.isInitializing() && PlatHelper.getPlatform().isForge()) {
            throw new AssertionError("Method has to be called during main mod initialization phase. Client and Server initializer are not valid, you must call in the main one");
        }
    }

    public static boolean isInitPhase() {
        return PlatHelper.isInitializing() || !PlatHelper.getPlatform().isForge();
    }

    public static void assertAfterInitPhase() {
        if (PlatHelper.isInitializing()) {
            throw new AssertionError("Method has to be called after main mod initialization phase. Client and Server initializer are not valid, you must call in the main one");
        }
    }

    @ApiStatus.Internal
    public static void addDependent(String modId) {
        if (!Set.of("minecraft", "neoforge", "fabric").contains(modId)) {
            DEPENDENTS.add(modId);
        }
    }

    //not ideal but works most of the times. this is populated when a mod invokes a ML registry function
    public static Set<String> getDependents() {
        return Set.copyOf(DEPENDENTS);
    }

    public static boolean isDependant(String modId) {
        return DEPENDENTS.contains(modId);
    }

    public static void crashIfInDev(String message) {
        if (PlatHelper.isDev()) throw new RuntimeException(message);
        else {
            Moonlight.LOGGER.error(message);
        }
    }

    public static void crashIfInDev() {
        crashIfInDev("");
    }

    public static void logIfInDev(String s) {
        if (PlatHelper.isDev()) LOGGER.error(s);
    }

    public static void registerBuiltinFluidBehavior(DispenserHelper.Event event) {
        Set<class_1792> itemSet = new HashSet<>();
        for (SoftFluid f : SoftFluidRegistry.get(event.getRegistryAccess())) {
            Collection<FluidContainerList.Category> categories = f.getContainerList().getCategories();
            for (FluidContainerList.Category c : categories) {
                class_1792 empty = c.getEmptyContainer();
                //prevents registering stuff twice
                if (empty != class_1802.field_8162 && !itemSet.contains(empty)) {
                    event.register(new DispenserHelper.FillFluidHolderBehavior(empty));
                    itemSet.add(empty);
                }
                for (class_1792 full : c.getFilledItems()) {
                    if (full != class_1802.field_8162 && !itemSet.contains(full)) {
                        event.register(new DispenserHelper.FillFluidHolderBehavior(full));
                        itemSet.add(full);
                    }
                }
            }
        }
    }

    public static void setVerboseLogging(boolean b) {
        verboseLogging = b;
    }

    public static boolean isVerboseLogging() {
        return verboseLogging;
    }
}
